@@ -16,7 +16,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2002-2004 Free Software Foundation
+# Copyright 2002-2005 Free Software Foundation
 #
 # FILE:
 # GFDisplayHandler.py
@@ -89,6 +89,8 @@ class FieldDisplayHandler(events.EventAware):
 
 
   def generateRefreshEvent(self):
+    gDebug (5, "generateRefreshEvent on %s '%s' %s" % \
+        (self, self.display, self.entry))
     # TODO: this should probably actually happen in UIwxpython!
     if (self.handleCR and type(self.display)=='str'):
       self.dispatchEvent(events.Event('updateEntryEditor',
@@ -106,7 +108,7 @@ class FieldDisplayHandler(events.EventAware):
          ))
 
   def isPending(self):
-    return self.modified
+    return self.editing and self.modified
 
   #####################
   #
@@ -121,7 +123,7 @@ class FieldDisplayHandler(events.EventAware):
     """
     Called when a widget first gets focus.
     """
-    self.editing = True
+    self.editing = self.field.isEditable(event._form.getCurrentMode())
     self.modified = False
 
     # TODO: Replace with formatter
@@ -167,7 +169,7 @@ class FieldDisplayHandler(events.EventAware):
            not self.field._allowedValues.has_key("%s" % self.value):
           self.work = ""
           event.__error__ = True
-          event.__errortext__ = _("Invalid value '%s' for field") % self.value
+          event.__errortext__ = u_("Invalid value '%s' for field") % self.value
           return
 
         self.editing = False
@@ -180,23 +182,16 @@ class FieldDisplayHandler(events.EventAware):
       else:
         self.dispatchEvent(events.Event('formALERT',
           u_("Invalid input: '%s'") % self.work,_form=event._form))
+    else:
+      self.editing = False
 
 
   def addText(self, event):
 
-    value = event.text
+    if not self.editing:
+      return
 
-    if (self.field.readonly and not self.field._block.mode=='query'):
-      event.__error__ = True
-      event.__errortext__ = _("Cannot modify field.  Form is read only")
-      return
-    elif self.field.editOnNull and \
-         self.value != None and \
-         self.value != "" and \
-         not self.field._block._resultSet.current.isPending():
-      event.__error__ = True
-      event.__errortext__ = _("Cannot modify field after initial creation.")
-      return
+    value = event.text
 
     # Do some appropriate checks on the text
     # TODO: This will be moved to the mask system!!
@@ -206,8 +201,8 @@ class FieldDisplayHandler(events.EventAware):
     if self.field._uppercase:
       value = string.upper(event.text)
 
-    if hasattr(self.field,'max_length') and \
-       len(self.work)  + len(value) > self.field.max_length:
+    if hasattr(self.field,'maxLength') and \
+       len(self.work)  + len(value) > self.field.maxLength:
       return
 
     if ( self.field._numeric and \
@@ -252,8 +247,17 @@ class FieldDisplayHandler(events.EventAware):
     self.modified = True
     self._buildDisplay()
 
+    # Update the field. This means PRE-CHANGE and POST-CHANGE will get fired
+    # now. For now, only do this here if we are a lookup.
+    if hasattr(self.field, 'fk_source'):
+      self._buildValue()
+      self.field.setValue(self.value)
+
   # Insert text at defined position
   def insertTextAt(self, event):
+    if not self.editing:
+      return
+
     # set cursor position
     self.cursor = event.position
 
@@ -262,13 +266,28 @@ class FieldDisplayHandler(events.EventAware):
 
   # Delete Text Range
   def deleteRange(self, event):
+    if not self.editing:
+      return
+
     self.selection1 = event.start_pos
     self.selection2 = event.end_pos
-    self.cursor = event.position
-    self.delete(event)
+    self.cursor     = event.position
+    self.delete (event)
+
+  # Handle <Enter> key for multiline entries
+  def __handleENTER(self, event):
+    if gConfigForms('EnterIsNewLine') and \
+       hasattr(self.entry, 'Char__height') and \
+       self.entry.Char__height > 1:
+      event.text = '\n'
+      self.addText(event)
+      event.drop()
 
   # Delete backwards one character
   def backspace(self, event):
+    if not self.editing:
+      return
+
 
     # If we have a "selection",
     # then act like a "delete"
@@ -287,6 +306,9 @@ class FieldDisplayHandler(events.EventAware):
 
   # Delete forward one character
   def delete(self, event):
+    if not self.editing:
+      return
+
     event.overstrike = True
     event.text = ""
 
@@ -377,10 +399,10 @@ class FieldDisplayHandler(events.EventAware):
 
   # Select the entire text of the entry and move
   # the cursor to the end
-  def selectAll(self, event):
+  def selectAll (self, event):
 
     self.selection1 = 0
-    self.moveCursorToEnd(event, True)
+    self.moveCursorToEnd (event, True)
     self.selection2 = self.cursor
 
 
@@ -426,21 +448,28 @@ class FieldDisplayHandler(events.EventAware):
 
   # Copy to the clipboard
   def clipboardCopy(self, event):
+
     if self.selection1 != None:
-      sel1, sel2 = self.getSelectionArea()
-      self.dispatchEvent(events.Event('setCLIPBOARD',
-               text=self.display[sel1:sel2]))
+      sel1, sel2 = self.getSelectionArea ()
+      self.dispatchEvent (events.Event ('setCLIPBOARD',
+               text = self.display [sel1:sel2]))
 
     event.refreshDisplay = False
 
 
   def clipboardCut(self, event):
+    if not self.editing:
+      return self.clipboardCopy(event)
+
     self.clipboardCopy(event)
     edevent = events.Event("requestKEYPRESS", text="", _form=event._form)
     self.dispatchEvent(edevent)
 
 
   def clipboardPaste(self, event):
+    if not self.editing:
+      return
+
     event.text = self.dispatchEvent(events.Event('getCLIPBOARD'))
     if event.text != None:
       self.addText(event)
@@ -450,7 +479,7 @@ class FieldDisplayHandler(events.EventAware):
   #
   # Internal methods
   #
-  def __init__(self, entry, eventHandler, formatter=None):
+  def __init__(self, entry, eventHandler, subEventHandler, formatter=None):
     events.EventAware.__init__(self, eventHandler)
 
     self.entry = entry            # Our entry
@@ -464,6 +493,7 @@ class FieldDisplayHandler(events.EventAware):
     self.selection2 = None        # End of highlight
     self.cursor = 0               # Cursor position
     self._loadedAllowedValues = False # the allowed values already been loaded
+    self.subEventHandler = subEventHandler
 
     # TODO: replace w/an event that asks the
     # TODO: UIdriver if this should happen!
@@ -472,11 +502,11 @@ class FieldDisplayHandler(events.EventAware):
     # Needs to be set by the child classes
     self.formatter = formatter
 
-    self.subevents = events.EventController()
-    self.subevents.registerEventListeners( {
+    self.subEventHandler.registerEventListeners( {
 
                  # "Entry" events
                  'requestKEYPRESS'     : self.addText,
+                 'requestENTER'        : self.__handleENTER,
                  'requestCURSORLEFT'   : self.moveCursorLeft,
                  'requestCURSORRIGHT'  : self.moveCursorRight,
                  'requestCURSOREND'    : self.moveCursorToEnd,
@@ -505,14 +535,42 @@ class FieldDisplayHandler(events.EventAware):
 
 
   def _buildValue(self):
-    self.value = self.work
+    if self.field._allowedValues:
+      if self.work == "":
+        self.value = '' # None
+      else:
+        if self.field._allowedValuesReverse.has_key (self.display):
+          self.value = self.field._allowedValuesReverse [self.display]
+        else:
+          self.value = None
+          return False
+    else:
+      self.value = self.work
     return True
 
+
   def _buildDisplayHelper(self, value, editing):
-    if value == None:
-      return ""
+    if self.field._allowedValues:
+      if editing:
+        val = value.lower ()
+
+        for disp in self.field._allowedValuesDescr:
+          if disp [:len (val)].lower () == val:
+            revVal = self.field._allowedValuesReverse [disp]
+            return self.field._allowedValues [revVal]
+
+        return value
+
+      if self.field._allowedValues.has_key ("%s" % value):
+        return self.field._allowedValues ["%s" % value]
+      else:
+        return ""
+
     else:
-      return "%s" % value
+      if value == None:
+        return ""
+      else:
+        return "%s" % value
 
   def _buildDisplay(self):
     if self.editing:
@@ -527,10 +585,12 @@ class FieldDisplayHandler(events.EventAware):
 #
 #
 class TextDisplayHandler(FieldDisplayHandler):
-  def __init__(self, entry, eventHandler, displayMask, inputMask):
+  def __init__(self, entry, eventHandler, subEventHandler, displayMask,
+               inputMask):
 ##    FieldDisplayHandler.__init__(self, entry, eventHandler,
 ##            TextMask.TextMask(displayMask, inputMask, formatMask))
-    FieldDisplayHandler.__init__(self, entry, eventHandler, None)
+    FieldDisplayHandler.__init__(self, entry, eventHandler, subEventHandler,
+                                 None)
 
 
 #############################################################################
@@ -538,10 +598,12 @@ class TextDisplayHandler(FieldDisplayHandler):
 #
 #
 class PasswordDisplayHandler(TextDisplayHandler):
-  def __init__(self, entry, eventHandler, displayMask, inputMask):
+  def __init__(self, entry, eventHandler, subEventHandler, displayMask,
+               inputMask):
 ##    FieldDisplayHandler.__init__(self, entry, eventHandler,
 ##            TextMask.TextMask(displayMask, inputMask, formatMask))
-    FieldDisplayHandler.__init__(self, entry, eventHandler, None)
+    FieldDisplayHandler.__init__(self, entry, eventHandler, subEventHandler,
+                                 None)
 
   #
   # beginEdit
@@ -551,7 +613,7 @@ class PasswordDisplayHandler(TextDisplayHandler):
     """
     Called when a widget first gets focus.
     """
-    self.editing = True
+    self.editing = self.field.isEditable(event._form.getCurrentMode())
     self.modified = False
 
     # TODO: Replace with formatter
@@ -562,8 +624,8 @@ class PasswordDisplayHandler(TextDisplayHandler):
     self.cursor = len(self.display)
     # Ensure cursor is properly placed.
     self.generateRefreshEvent()
-    
-    
+
+
   def _buildDisplayHelper(self, value, editing):
     if value == None:
       return ""
@@ -583,17 +645,19 @@ class PasswordDisplayHandler(TextDisplayHandler):
 # Handler for Numeric types
 #
 class NumberDisplayHandler(FieldDisplayHandler):
-  def __init__(self, entry, eventHandler, displayMask, inputMask):
+  def __init__(self, entry, eventHandler, subEventHandler, displayMask,
+               inputMask):
 ##    FieldDisplayHandler.__init__(self, entry, eventHandler,
 ##            DateMask.DateMask(displayMask, inputMask, displayMask))
-    FieldDisplayHandler.__init__(self, entry, eventHandler, None)
+    FieldDisplayHandler.__init__(self, entry, eventHandler, subEventHandler,
+                                 None)
 
 
   def setValue(self, value):
     if value in (None,""):
       value = None
     else:
-      value = float(value)
+      value = value
 
     return FieldDisplayHandler.setValue(self, value)
 
@@ -609,19 +673,20 @@ class NumberDisplayHandler(FieldDisplayHandler):
     if editing and type(val) == types.StringType:
       return val
 
+#     if float(val) == val:
+#    try:
+    value = val
     try:
-      value = float(val)
-    except ValueError:
-      return str(val)
-
-    try:
-      if int(value) == value:
-        value = "%d" % value
-      else:
-        value = "%s" % value
+        if int(value) == value:
+          value = "%d" % value
+        else:
+          value = "%s" % value
     except OverflowError:
-      value = "%s" % value
-
+        value = "%s" % value
+    except ValueError:
+        value = "%s" % value
+#     else:
+#       value = "%s" % val
 #     if not editing:
 #       # Comma-fy (Temporary!)
 #       if value[:1] == '-':
@@ -652,7 +717,8 @@ class NumberDisplayHandler(FieldDisplayHandler):
       return True
 
     try:
-      self.value = float(self.work)
+      dummy = float(self.work)
+      self.value = self.work
       return True
     except ValueError:
       return False
@@ -665,10 +731,12 @@ class NumberDisplayHandler(FieldDisplayHandler):
 # Handler for Date types
 #
 class DateDisplayHandler(FieldDisplayHandler):
-  def __init__(self, entry, eventHandler, displayMask, inputMask):
+  def __init__(self, entry, eventHandler, subEventHandler, displayMask,
+               inputMask):
 ##    FieldDisplayHandler.__init__(self, entry, eventHandler,
 ##            DateMask.DateMask(displayMask, inputMask, displayMask))
-    FieldDisplayHandler.__init__(self, entry, eventHandler, None)
+    FieldDisplayHandler.__init__(self, entry, eventHandler, subEventHandler,
+                                 None)
     self.__displayMask = displayMask
     self.__inputMask = inputMask
 
@@ -750,11 +818,105 @@ class DropdownDisplayHandler(FieldDisplayHandler):
     FieldDisplayHandler.__init__(self, *args, **params)
 
     # My events...
-    self.subevents.registerEventListeners( {
+    self.subEventHandler.registerEventListeners( {
            'requestREPLACEVALUE' : self.replaceText,
            'requestCOMBODROPPED' : self.beginEdit } )
 
 
+  def beginEdit(self, event):
+
+    if self.editing == True and self.modified == False:
+      return
+
+    if not self._loadedAllowedValues and not hasattr(self.field,'_allowedValues'):
+      self.field.allowedValues()
+      self._loadedAllowedValues = True
+
+    self.editing = self.field.isEditable(event._form.getCurrentMode())
+    self.modified = False
+
+    # TODO: Replace with formatter
+    self.setValue(self.field.getValue())
+
+    if self.value == None:
+      self.work = ""
+    else:
+      try:
+        self.work = self.field._allowedValues ["%s" % self.value]
+      except KeyError:
+        self.work = ""
+        event.__error__ = True
+        event.__errortext__ = u_("Invalid value '%s' for keyed pull-down "
+                                 "field") % self.value
+
+    self._buildDisplay()
+
+    self.cursor = len(self.display)
+
+
+
+  # TODO: Replace with format mask
+  def _buildDisplayHelper(self, value, editing):
+    if value in (None,""):
+      return ""
+
+    if editing:
+      val = string.lower(value)
+      for disp in self.field._allowedValuesDescr:
+        if string.lower(disp[:len(val)]) == val:
+          display = self.field._allowedValues[self.field._allowedValuesReverse[disp]]
+          return display
+      return value
+
+    if self.field._allowedValues.has_key ("%s" % value):
+      return self.field._allowedValues ["%s" % value]
+    else:
+      return ""
+
+  def _buildDisplay(self):
+    if self.editing:
+      self.display = self._buildDisplayHelper(self.work, True)
+      if self.cursor > len(self.work):
+        self.work = self.display
+    else:
+      self.display = self._buildDisplayHelper(self.value, False)
+
+
+  def replaceText (self, event):
+    if not self.editing:
+      return
+
+    self.selection1 = 0
+    self.selection2 = len (self.display)
+    self.addText (event)
+
+# TODO: Kills dropdown handling with keyboard on win32
+# TODO: needs more work :(
+#    self.endEdit(event) # Hack: allows dropdowns to change field
+                        # value immediately (causes post-change
+                        # on the field to fire).
+                        # I don't think it'll hurt anything.
+                        # if it does then we'll need to pull
+                        # the logic from FieldDisplayHandler.endEdit
+                        # and add here.  - jamest
+
+
+
+#############################################################################
+#
+# Handler for Listbox types
+#
+class ListboxDisplayHandler(FieldDisplayHandler):
+
+  def __init__(self, *args, **params):
+
+    FieldDisplayHandler.__init__(self, *args, **params)
+
+    # My events...
+    self.subEventHandler.registerEventListeners( {
+           'requestREPLACEVALUE' : self.replaceText } )
+
+
   def beginEdit(self, event):
     if self.editing == True and self.modified == False:
       return
@@ -777,13 +939,13 @@ class DropdownDisplayHandler(FieldDisplayHandler):
       except KeyError:
         self.work = ""
         event.__error__ = True
-        event.__errortext__ = _("Invalid value '%s' for keyed pull-down field") % self.value
+        event.__errortext__ = u_("Invalid value '%s' for keyed pull-down "
+                                 "field") % self.value
 
     self._buildDisplay()
 
     self.cursor = len(self.display)
 
-
   # TODO: Replace with format mask
   def _buildDisplayHelper(self, value, editing):
     if value in (None,""):
@@ -816,27 +978,6 @@ class DropdownDisplayHandler(FieldDisplayHandler):
     self.selection2 = len(self.display)
     self.addText(event)
 
-# TODO: Kills dropdown handling with keyboard on win32
-# TODO: needs more work :(
-#    self.endEdit(event) # Hack: allows dropdowns to change field
-                        # value immediately (causes post-change
-                        # on the field to fire).
-                        # I don't think it'll hurt anything.
-                        # if it does then we'll need to pull
-                        # the logic from FieldDisplayHandler.endEdit
-                        # and add here.  - jamest
-
-
-  def _buildValue(self):
-    if self.work == "":
-      self.value = '' # None
-    else:
-      try:
-        self.value = self.field._allowedValuesReverse[self.display]
-      except KeyError:
-        return False
-    return True
-
 
 
 #############################################################################
@@ -856,8 +997,8 @@ class CheckboxDisplayHandler(FieldDisplayHandler):
     FieldDisplayHandler.__init__(self, *args, **params)
 
     # My events...
-    self.subevents.registerEventListeners( {
-           'requestTOGGLECHKBOX' : self.toggle  } )
+    self.subEventHandler.registerEventListeners( {
+           'requestTOGGLECHKBOX' : self.handleToggleChkbox} )
 
 
   def setValue(self, value):
@@ -885,24 +1026,37 @@ class CheckboxDisplayHandler(FieldDisplayHandler):
   def _buildDisplayHelper(self, value, editing):
     return self._sanitizeValue(value)
 
-##  def getDisplay(self):
-##    return self.display
 
+  # Helpers for user events:
 
-  def toggle(self, event):
-    if event.data == None:
-      # saving our checkboxes from Python 2.3 boolean type (it kills sql update)
-      self.work = int(not self.work)
-    else:
-      self.work = self._sanitizeValue(event.data[1])
+  # Set checkbox to boolean value
+  def __set (self, value):
+    if value != self.work:
+      self.work = value
+      self.modified = True
+      self._buildDisplay ()
+
+  # Toggle value of checkbox
+  def __toggle (self):
+    self.work = not self.work
     self.modified = True
-    self._buildDisplay()
-    event.refreshDisplay=True
+    self._buildDisplay ()
+
+
+  # Handle requestTOGGLECHKBOX event
+  def handleToggleChkbox (self, event):
+    if not self.editing:
+      return
+    if event.data == None:
+      self.__toggle ()
+    else:
+      self.__set (self._sanitizeValue (event.data [1]))
+    event.refreshDisplay = True
 
 
   def beginEdit(self, event):
 
-    self.editing = True
+    self.editing = self.field.isEditable(event._form.getCurrentMode())
     self.modified = False
 
     self.setValue(self.field.getValue())
@@ -914,6 +1068,17 @@ class CheckboxDisplayHandler(FieldDisplayHandler):
     self.cursor = 0
 
 
+  # Correctly handle requestKEYPRESS event
+  def addText(self, event):
+    if event.text == ' ':
+      self.__toggle ()
+    elif event.text in ['0', '-']:      # TODO: add "Y" for current language
+      self.__set (False)
+    elif event.text in ['1', '+']:      # TODO: add "N" for current language
+      self.__set (True)
+    event.refreshDisplay = True
+    return
+
   # Delete backwards one character
   def backspace(self, event):
     return
@@ -998,6 +1163,9 @@ class CheckboxDisplayHandler(FieldDisplayHandler):
 
 
   def clipboardPaste(self, event):
+    if not self.editing:
+      return
+
     event.text = self.dispatchEvent(events.Event('getClipboard'))
     if event.text != None:
       self.work = self._sanitizeValue(event.data)
@@ -1011,10 +1179,11 @@ class CheckboxDisplayHandler(FieldDisplayHandler):
 # ImageDiplayHandler
 #
 class ImageDisplayHandler(FieldDisplayHandler):
-  def __init__(self, entry, eventHandler):
+  def __init__(self, entry, eventHandler, subEventHandler):
     if not Image:
       raise "Form contains a <image> and image support not loaded"
-    FieldDisplayHandler.__init__(self, entry, eventHandler, None)
+    FieldDisplayHandler.__init__(self, entry, eventHandler, subEventHandler,
+                                 None)
 
   def _buildDisplayHelper(self, value, editing):
     if string.lower(self.entry.type) == 'url':
@@ -1033,8 +1202,9 @@ class ImageDisplayHandler(FieldDisplayHandler):
 # ComponentDiplayHandler
 #
 class ComponentDisplayHandler(FieldDisplayHandler):
-  def __init__(self, entry, eventHandler):
-    FieldDisplayHandler.__init__(self, entry, eventHandler, None)
+  def __init__(self, entry, eventHandler, subEventHandler):
+    FieldDisplayHandler.__init__(self, entry, eventHandler, subEventHandler,
+                                 None)
 
   def _buildDisplayHelper(self, value, editing):
     if string.lower(self.entry.type) == 'url':
