@@ -1,5 +1,8 @@
-# #
-# This file is part of GNU Enterprise.
+# GNU Enterprise Forms - GF Class Hierarchy - Block
+#
+# Copyright 2001-2005 Free Software Foundation
+#
+# This file is part of GNU Enterprise
 #
 # GNU Enterprise is free software; you can redistribute it
 # and/or modify it under the terms of the GNU General Public
@@ -16,23 +19,14 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2000-2004 Free Software Foundation
-#
-# FILE:
-# GFBlock.py
-#
-# DESCRIPTION:
+# $Id: GFBlock.py 7090 2005-03-04 17:47:02Z reinhard $
 """
 Classes making up the Block object
 """
-# NOTES:
-#
-# HISTORY:
-#
 
 from gnue.forms.GFObjects.GFDataSource import GFDataSource
 
-from gnue.common.apps import GDebug
+from gnue.common.apps import errors
 from gnue.common.datasources import GConditions
 from GFContainer import GFContainer
 from gnue.common import events
@@ -53,9 +47,9 @@ class GFBlock(GFContainer, events.EventAware):
     GFContainer.__init__(self, parent, 'GFBlock')
 
     self.mode = 'normal'
-    self._convertAstericksToPercent = gConfigForms('AsterickWildcard')
+    self._convertAsterisksToPercent = gConfigForms('AsteriskWildcard')
 
-    self._inits = [self.initialize]
+    self._inits = [self.__initialize]
 
     #self._datasource = None
     self._resultSet = None
@@ -66,7 +60,7 @@ class GFBlock(GFContainer, events.EventAware):
     self._queryDefaults = {}
     self._queryValues = {}
     self._lastQueryValues = {}
-    self._autocreate =  int(gConfigForms('autocreate'))
+
     self._gap = 0
     self._rows = 1
 
@@ -103,50 +97,74 @@ class GFBlock(GFContainer, events.EventAware):
 
 
     self._triggerGlobal = 1
-    self._triggerFunctions={'clear':{'function':self.processClear,
-                                     'description':''},
-                            'commit':{'function':self.commit,
-                                      'description':''},
-                            'deleteRecord':{'function':self.deleteRecord,
-                                            'description':''},
-                            'gotoRecord':{'function':self.jumpRecord,
-                                          'description':''},
-                            'firstRecord':{'function':self.firstRecord,
-                                           'description':'Navigates the block to the first record it contains.'},
-                            'isEmpty':{'function':self.isEmpty,
-                                       'description':'Returns True if block is empty.'},
-                            'isSaved':{'function':self.isSaved,
-                                       'description':'Returns True if block contains no modified records.'},
-                            'lastRecord':{'function':self.lastRecord,
-                                      'description':'Navigates the block to the last record it contains.'},
-                            'newRecord':{'function':self.newRecord,
-                                         'description':''},
-                            'nextRecord':{'function':self.nextRecord,
-                                          'description':'Navigates the block to the next record in sequence.'},
-                            'prevRecord':{'function':self.prevRecord,
-                                          'description':'Navigates the block to the previous record in sequence.'},
-                            'jumpRecords':{'function':self.jumpRecords,
-                                          'description':'Navigates the specified number of records.'},
-                            'rollback':{'function':self.processRollback,
-                                        'description':'Clears all records regardless of state from the block'},
-                            'initQuery':{'function':self.initQuery,
-                                        'description':'Prepares the block for query input.'},
-                            'copyQuery':{'function':self.copyQuery,
-                                        'description':'Prepares the block for query input.'},
-                            'cancelQuery':{'function':self.cancelQuery,
-                                        'description':'Cancels query input.'},
-                            'executeQuery':{'function':self.processQuery,
-                                            'description':'Executes the current query.'},
-                            }
+    self._triggerFunctions={
+        'clear':{'function':self.processClear,
+                  'description':''},
+        'getResultSet':{'function':self.getResultSet,
+                  'description':''},
+        'commit':{'function':self.commit,
+                  'description':''},
+        'newRecord':{'function':self.newRecord,
+                      'description':''},
+        'deleteRecord':{'function':self.deleteRecord,
+                        'description':''},
+        'duplicateRecord':{'function':self.duplicateRecord,
+                        'description':'Duplicates the current (non-empty) record into a new record.'},
+        'gotoRecord':{'function':self.jumpRecord,
+                      'description':''},
+        'firstRecord':{'function':self.firstRecord,
+                        'description':'Navigates the block to the first record it contains.'},
+        'isEmpty':{'function':self.isEmpty,
+                    'description':'Returns True if block is empty.'},
+        'isSaved':{'function':self.isSaved,
+                    'description':'Returns True if block contains no modified records.'},
+        'lastRecord':{'function':self.lastRecord,
+                  'description':'Navigates the block to the last record it contains.'},
+        'nextRecord':{'function':self.nextRecord,
+                      'description':'Navigates the block to the next record in sequence.'},
+        'prevRecord':{'function':self.prevRecord,
+                      'description':'Navigates the block to the previous record in sequence.'},
+        'jumpRecords':{'function':self.jumpRecords,
+                      'description':'Navigates the specified number of records.'},
+        'rollback':{'function':self.processRollback,
+                    'description':'Clears all records regardless of state from the block'},
+        'initQuery':{'function':self.initQuery,
+                    'description':'Prepares the block for query input.'},
+        'copyQuery':{'function':self.copyQuery,
+                    'description':'Prepares the block for query input.'},
+        'cancelQuery':{'function':self.cancelQuery,
+                    'description':'Cancels query input.'},
+        'executeQuery':{'function':self.processQuery,
+                        'description':'Executes the current query.'},
+        'call': {'function'   : self.callFunction,
+                 'description': 'Executes a function of the datasource'},
+        'update': {'function': self.updateCurrentRecordSet}
+        }
+
+    self._triggerProperties={
+          'parent':        {'get':self.getParent},
+          'autoCommit':    {'set':self.triggerSetAutoCommit,
+                            'get':self.triggerGetAutoCommit },
+          'queryable':     {'set':self.triggerSetQueryable,
+                            'get':self.triggerGetQueryable },
+          'editable':      {'set':self.triggerSetEditable,
+                            'get':self.triggerGetEditable },
+          'autoCreate':    {'set':self.triggerSetAutoCreate,
+                            'get':self.triggerGetAutoCreate },
+          'deletable':     {'set':self.triggerSetDeletable,
+                            'get':self.triggerGetDeletable },
+          'transparent':   {'set':self.triggerSetTransparent,
+                            'get':self.triggerGetTransparent },
+          'autoNextRecord':{'set':self.triggerSetAutoNextRecord,
+                            'get':self.triggerGetAutoNextRecord },
+       }
 
-    self._triggerProperties={'parent':  {'get':self.getParent}}
 
-     
   # Iterator support (Python 2.2+)
   def __iter__(self):
     """
     Iterator support for Python 2.2+
-    
+
     Allows you to do:
       for foo in myBlock:
         ....
@@ -163,6 +181,14 @@ class GFBlock(GFContainer, events.EventAware):
     if hasattr(self, 'rowSpacer'):
       self._gap = self.rowSpacer
 
+    if hasattr(self, 'restrictDelete') and self.restrictDelete:
+      self.deletable = False
+      del self.__dict__['restrictDelete']
+
+    if hasattr(self, 'restrictInsert') and self.restrictInsert:
+      self.editable = 'update'
+      del self.__dict__['restrictInsert']
+
     if hasattr(self,'datasource'):
       self.datasource = string.lower(self.datasource)
 
@@ -171,7 +197,7 @@ class GFBlock(GFContainer, events.EventAware):
   #
   # Primary initialization
   #
-  def initialize(self):
+  def __initialize(self):
     self._form = form = self.findParentOfType('GFForm')
     self._logic = logic = self.findParentOfType('GFLogic')
 
@@ -183,18 +209,30 @@ class GFBlock(GFContainer, events.EventAware):
     # Initialize our events system
     events.EventAware.__init__(self, form._instance.eventController)
 
-    # Get all focusable items, ordered correctly
-    self._focusOrder = self.getFocusOrder()
-
     # Create a stub/non-bound datasource if we aren't bound to one
     if not hasattr(self,'datasource') or not self.datasource:
-      ds = GFDataSource(self)
+      ds = GFDataSource(self._form)
       self.datasource = ds.name = "__dts_%s" % id(self)
       form._datasourceDictionary[ds.name] = ds
       ds._buildObject()
       ds.phaseInit()
 
-    self._dataSourceLink = form._datasourceDictionary[self.datasource]
+    self._dataSourceLink = form._datasourceDictionary.get (self.datasource)
+    if self._dataSourceLink is None:
+      raise errors.ApplicationError, \
+          u_("Datasource '%(datasource)s' in block '%(block)s' not found") \
+          % {'datasource': self.datasource,
+             'block': self.name}
+
+    # Get min and max child rows, if applicable
+    try:
+      self._minChildRows = self._dataSourceLink.detailmin
+    except AttributeError:
+      self._minChildRows = 0
+    try:
+      self._maxChildRows = self._dataSourceLink.detailmax
+    except AttributeError:
+      self._maxChildRows = None
 
     # We will monitor our own resultSet changes
     self._dataSourceLink.registerResultSetListener(self._loadResultSet)
@@ -212,41 +250,83 @@ class GFBlock(GFContainer, events.EventAware):
     else:
       object._gap = self._gap
 
+  #
+  #
+  #
+  def getFocusOrder(self):
+    list = []
+    for field in self._children:
+      try:
+        list += field._entryList
+      except AttributeError:
+        pass # Triggers, etc
+
+    return GFContainer.getFocusOrder(self,list)
 
   #
   # isSaved
   #
-  # returns a positive value if the datasource the block is associated
-  # with is saved
   #
   def isSaved(self):
-    return self._resultSet == None or not self._resultSet.isPending()
+    """
+    Returns True if the datasource the block is associated
+    with is not pending any changes.
+    """
+    # TODO: Find all uses of isSaved() and replace with: not isPending()
+    return not self.isPending()
+
+  def isPending(self):
+    """
+    Returns True if the datasource the block is associated
+    with is pending any changes.
+    """
+    return self._resultSet and self._resultSet.isPending()
 
   #
   # deleteRecord
   #
-  # Doesn't really delete the record but marks it for
-  # deletion during next commit
-  #
   def deleteRecord(self):
+    """
+    Doesn't really delete the record but marks it for
+    deletion during next commit
+    """
     self._resultSet.current.delete()
 
   #
-  # Load and process a resultset
   #
   def _loadResultSet(self, resultSet):
+    """
+    Load and process a resultset
+    """
     self._resultSet = resultSet
     self._currentRecord = -1
     self._recordCount = 0
+
+    # This makes the resultSet call our currentRecordMoved method
+    resultSet.registerListener (self)
+
+    # FIXME: Is this used anywhere?
     resultSet._block = self
 
     if self._resultSet.firstRecord():
-      self.switchRecord(0)
+      # self.switchRecord(0)            # happens via currentRecordMoved
       self._recordCount = self._resultSet.getRecordCount()
     else:
-      # TODO: This means no results were returned from a query.
-      # TODO: It probably shouldn't create a new record :)  Display a message?
-      self.newRecord()
+      # This means no results were returned from a query.
+# #       if self._minChildRows:
+# #         # If datasource.detailmin is set, create some blank
+# #         # (but non-empty) records
+# #         while self._recordCount < self._minChildRows:
+# #           self.newRecord()
+# #           # Force the record to no longer be
+# #           field = self._fieldList[0]
+# #           field.setValue(field.getValue())
+# #         self.firstRecord()
+# #       else:
+        # Otherwise, give us a single empty record to work from
+        self.newRecord()
+
+
 
   #
   # isEmpty()
@@ -254,14 +334,27 @@ class GFBlock(GFContainer, events.EventAware):
   def isEmpty(self):
     return self._resultSet.current.isEmpty()
 
+  #
+  # getResultSet()
+  #
+  def getResultSet(self):
+    return self._resultSet
+
+
+  # This gets called by the resultSet whenever the current record changes
+  def currentRecordMoved (self):
+    self.switchRecord (self._resultSet.getRecordNumber() - self._currentRecord)
+
   #
   #
-  # Moves the proper record into editing position
   #
   def switchRecord(self, adjustment):
-
+    """
+    Moves the proper record into editing position
+    """
     self._currentRecord = self._resultSet.getRecordNumber()
     for field in self._fieldList:
+      field.gotNewCurrentRecord()
       for entry in field._entryList:
         # This loop is probably better somewhere else
         entry.recalculateVisible( adjustment, self._currentRecord, self._recordCount)
@@ -281,6 +374,8 @@ class GFBlock(GFContainer, events.EventAware):
 
     # Focus out
     self.processTrigger('PRE-FOCUSOUT')
+    if self.autoCommit and self._resultSet.current:
+      self._form.commit()
     self.processTrigger('POST-FOCUSOUT')
 
     if self._resultSet.insertRecord():
@@ -291,32 +386,45 @@ class GFBlock(GFContainer, events.EventAware):
           self._resultSet.current.setField(field, value, 0)
 
       self._recordCount = self._resultSet.getRecordCount()
-      self.switchRecord(1)
-
+      # self.switchRecord(1)            # happens via currentRecordMoved
       # Focus in
       self.processTrigger('PRE-FOCUSIN')
       self.processTrigger('ON-NEWRECORD')
+
       self.processTrigger('POST-FOCUSIN')
 
+  def duplicateRecord(self, exclude=(), include=()):
+    self._resultSet.duplicateRecord(exclude=exclude, include=include)
+
+  def isLastRecord(self):
+    return self._resultSet.isLastRecord()
+
+  def isFirstRecord(self):
+    return self._resultSet.isFirstRecord()
+
   def nextRecord(self):
     if not self._resultSet.isLastRecord():
 
       # Do FocusOut triggers
       self.processTrigger('PRE-FOCUSOUT')
-      # TODO: I'm not 100% sure of the definition of
-      # TODO: Post-FocusOut  In this context!
+      if self.autoCommit:
+        self._form.commit()
       self.processTrigger('POST-FOCUSOUT')
 
       self._resultSet.nextRecord()
       self._recordCount = self._resultSet.getRecordCount()
-      self.switchRecord(1)
+      # self.switchRecord(1)            # happens via currentRecordMoved
 
       # Focus in
       self.processTrigger('PRE-FOCUSIN')
       self.processTrigger('POST-FOCUSIN')
 
-    elif self._autocreate and not self.isEmpty() and not self.restrictInsert:
+    elif self.autoCreate and not self.isEmpty() and not self.editable in ('update','N'):
       self.newRecord()
+      # Go to first field
+      self._form.findAndChangeFocus(self._entryList[0])
+
+
 
   def lastRecord(self):
     if not self._resultSet.isLastRecord():
@@ -325,7 +433,8 @@ class GFBlock(GFContainer, events.EventAware):
       self.processTrigger('POST-FOCUSOUT')
 
       self._resultSet.lastRecord()
-      self.switchRecord(0)
+      # jump = self._resultSet.getRecordNumber () - self._currentRecord
+      # self.switchRecord (jump)        # happens via currentRecordMoved
 
       # Focus in
       self.processTrigger('PRE-FOCUSIN')
@@ -338,7 +447,7 @@ class GFBlock(GFContainer, events.EventAware):
       self.processTrigger('POST-FOCUSOUT')
 
       self._resultSet.firstRecord()
-      self.switchRecord(0)
+      # self.switchRecord(0)            # happens via currentRecordMoved
 
       # Focus in
       self.processTrigger('PRE-FOCUSIN')
@@ -352,24 +461,27 @@ class GFBlock(GFContainer, events.EventAware):
       self.processTrigger('POST-FOCUSOUT')
 
       self._resultSet.prevRecord()
-      self.switchRecord(-1)
+      # self.switchRecord(-1)           # happens via currentRecordMoved
 
       # Focus in
       self.processTrigger('PRE-FOCUSIN')
       self.processTrigger('POST-FOCUSIN')
 
   def commit(self):
+    print "DEPRECIATION WARNING: the use of block.commit () is depreciated.", \
+          "Please use form.commit () instead."
+    self._form.commit ()
+
     #Commented out (dimas)
     #Does not work properly anyway
     #self._form.changeFocus(self)
     #self._form.commit()
-    
+
     #TODO: Add error handling
     #TODO: Check how triggers performed
     #TODO: original code is in GFForm.commit
-    self._precommitRecord = self._currentRecord
-
-    self.processCommit()
+    #self._precommitRecord = self._currentRecord
+    #self.processCommit()
 
   def jumpRecord(self, recordNumber):
     # If recordNumber is negative, move relative to last record
@@ -377,7 +489,7 @@ class GFBlock(GFContainer, events.EventAware):
       recordNumber += self._resultSet.getRecordCount()
     if recordNumber < 0:
       raise "Invalid record number"
-    
+
     if recordNumber != self._resultSet.getRecordNumber():
       # Focus out
       self.processTrigger('PRE-FOCUSOUT')
@@ -386,36 +498,46 @@ class GFBlock(GFContainer, events.EventAware):
       if not self._resultSet.setRecord(recordNumber):
         self._resultSet.lastRecord()
 
-      jump = self._resultSet.getRecordNumber() - self._currentRecord
-      self._currentRecord = self._resultSet.getRecordNumber()
-      self.switchRecord(jump)
+      # jump = self._resultSet.getRecordNumber() - self._currentRecord
+      # self._currentRecord = self._resultSet.getRecordNumber()
+      # self.switchRecord(jump)         # happens via currentRecordMoved
 
       # Focus in
       self.processTrigger('PRE-FOCUSIN')
       self.processTrigger('POST-FOCUSIN')
 
-      
+
   def jumpRecords(self, adjustment):
     targetRecord = self._resultSet.getRecordNumber() + adjustment
-    
+
     if targetRecord < 0:
       targetRecord = 0
     elif targetRecord > self._resultSet.getRecordCount():
       targetRecord = self._resultSet.getRecordCount()
 
     self.jumpRecord(targetRecord)
-    self._form._instance.updateRecordCounter(self._form) 
-  
+    self._form._instance.updateRecordCounter(self._form)
+
   #
   # processCommit
   #
   def processCommit(self):
-    GDebug.printMesg(1, "processing commit on block %s"%self.name,1)
+    gDebug (4, "processing commit on block %s"%self.name,1)
 
     self.mode='commit'
-    
+
     self._resultSet.setRecord(self._precommitRecord)
 
+    # Step thru all the records looking for records with uncommited 
+    # changes that had default values set.  If one is found then those
+    # fields are reset to their current values *with* modification tracking
+    # on.  The default population on new records is to set the value without
+    # marking the record as modified which prevents their value from being saved.
+    for record in self._resultSet._cachedRecords:
+      if record.isPending():
+        for field in self._lastValues.keys():
+          record.setField(field, record.getField(field), 1)
+
     # Backstep thru the record adjusting for any prior records that
     # will be deleted.  Keeps the record postition properly adjusted
     # during deletes.
@@ -428,23 +550,23 @@ class GFBlock(GFContainer, events.EventAware):
 
     if not self._dataSourceLink.hasMaster():
       self._resultSet.post()
-      self._dataSourceLink._dataObject.commit()
+      # The real commit will be done by the form once per connection
+      # self._dataSourceLink._dataObject.commit()
 
+  #
+  # Called after the commit on the backend is through.
+  #
+  def finalizeCommit (self):
+
+    # Our recordCount might have changed if records were deleted
     self._recordCount = self._resultSet.getRecordCount()
 
-    # Make sure the block is aware of the current
-    # position in the records.  If this is removed
-    # ui corruption can occur in zipcode.gfd if
-    # you commit while in or after a record that
-    # is to be deleted.
-
-    self.jumpRecord(self._precommitRecord)
-
-    # If all our records were deleted, create an empty record
+    # If all records were deleted, create an empty new one, otherwise make sure
+    # we move the record pointer to the correct record
     if not self._recordCount:
       self.newRecord()
     else:
-      self.jumpRecord(self._resultSet.getRecordNumber())
+      self.jumpRecord(self._precommitRecord)
 
     self.mode='normal'
 
@@ -460,6 +582,7 @@ class GFBlock(GFContainer, events.EventAware):
       self._dataSourceLink.createEmptyResultSet(masterRecordSet=self._resultSet._masterRecordSet)
     self._currentRecord = 0
     self._recordCount = 0
+    # FIXME: does this happen via currentRecordMoved?
     self.switchRecord(0)
 
 
@@ -470,13 +593,28 @@ class GFBlock(GFContainer, events.EventAware):
   # if recovering=True, then a commit or such failed and we need to clean up
   # (but not lose state information)
   #
-  def processRollback(self, recovering=False):
+  def processRollback (self, recovering = False, backendRollback = True):
+
+    # if called from GFForm the connection's rollback () has been executed
+    # already. But if called from a trigger code we do it here
+    if backendRollback:
+      if self._dataSourceLink._dataObject._connection is not None:
+        self._dataSourceLink._dataObject._connection.rollback ()
+
     if not recovering:
       self._currentRecord = 0
-      self._recordCount = 0
-      self._dataSourceLink.createEmptyResultSet()
-    self._dataSourceLink._dataObject.rollback()
-    self.switchRecord(0)
+      self._recordCount   = 0
+
+      if not self._dataSourceLink.hasMaster ():
+        self._dataSourceLink.createEmptyResultSet ()
+      else:
+        # if we have a master make sure to keep in touch with it
+        self._dataSourceLink.createEmptyResultSet ( \
+            masterRecordSet = self._resultSet._masterRecordSet)
+
+    # FIXME: does this happen via currentRecordMoved?
+    self.switchRecord (0)
+
 
   #
   # initQuery and processQuery
@@ -494,12 +632,12 @@ class GFBlock(GFContainer, events.EventAware):
         self._queryValues.update(self._queryDefaults)
         self.switchRecord(0)
 
-  def copyQuery(self): 
+  def copyQuery(self):
     self._query2 = 0
     self._queryValues = {}
     self._queryValues.update(self._lastQueryValues)
     self.switchRecord(0)
-  
+
   def cancelQuery(self):
     self.mode = 'normal'
     self.switchRecord(0)
@@ -574,6 +712,81 @@ class GFBlock(GFContainer, events.EventAware):
   def registerScrollbar(self, sb):
     self._scrollbars.append(sb)
 
+  # ---------------------------------------------------------------------------
+  # Call a datasource's function
+  # ---------------------------------------------------------------------------
+
+  def callFunction (self, name, parameters):
+    try:
+      res = self._dataSourceLink.callFuncOfCurrentRecordsetEntry (name,
+                        parameters)
+    finally:
+      self.switchRecord (0)
+
+    return res
+
+
+  # ---------------------------------------------------------------------------
+  # Update the current datasource's record set
+  # ---------------------------------------------------------------------------
+
+  def updateCurrentRecordSet (self):
+    try:
+      self._dataSourceLink.updateCurrentRecordSet ()
+
+    finally:
+      self.switchRecord (0)
+
+
+  ###################################################################
+  #
+  # Trigger settable stuff
+  #
+  def triggerSetAutoCommit(self, value):
+    self.autoCommit = not not value # Force boolean
+
+  def triggerGetAutoCommit(self):
+    return self.autoCommit
+
+  def triggerSetQueryable(self, value):
+    self.queryable = not not value # Force boolean
+
+  def triggerGetQueryable(self):
+    return self.queryable
+
+  def triggerSetEditable(self, value):
+    assert (value in ('Y','new','update','N'))
+    self.editable = value
+
+  def triggerGetEditable(self):
+    return self.editable
+
+  def triggerSetAutoCreate(self, value):
+    self.autoCreate = not not value # Force boolean
+
+  def triggerGetAutoCreate(self):
+    return self.autoCreate
+
+  def triggerSetDeletable(self, value):
+    self.deletable = not not value # Force boolean
+
+  def triggerGetDeletable(self):
+    return self.deletable
+
+  def triggerSetTransparent(self, value):
+    self.transparent = not not value # Force boolean
+
+  def triggerGetTransparent(self):
+    return self.transparent
+
+  def triggerSetAutoNextRecord(self, value):
+    self.autoNextRecord = not not value # Force boolean
+
+  def triggerGetAutoNextRecord(self):
+    return self.autoNextRecord
+
+
+
 #
 # _generateConditional
 #
@@ -587,19 +800,16 @@ def _generateConditional(block):
     for entry, val in block._queryValues.items():
         if entry._bound and entry.isQueryable() and len(str(val)):
           if entry.typecast == 'text':
-            if block._convertAstericksToPercent:
-              print val
+            if block._convertAsterisksToPercent:
               try:
                 val = str(val).replace('*','%')
               except ValueError:
                 pass
-            print val, val.find('%'), val.find('_')
             if (val.find('%') >= 0 or val.find('_') >= 0):
               conditionLike[entry.field] = val
             else:
               conditionEq[entry.field] = val
           else:
-##            GDebug.printMesg(5,'Adding conditional AND (%s=%s)' % (entry.field, val))
             conditionEq[entry.field] = val
 
     if len(conditionLike.keys()) and len(conditionEq.keys()):
@@ -622,6 +832,7 @@ def _generateConditional(block):
 
 
 
+
 # A simple resultset iterator
 # Lets you use ResultSets as:
 #
@@ -651,10 +862,10 @@ class _BlockIter:
       raise StopIteration
     else:
       self.block.nextRecord()
-    
-    if self.block.isEmpty() and self.block._resultSet.isLastRecord(): 
+
+    if self.block.isEmpty() and self.block._resultSet.isLastRecord():
       self.done = True
       raise StopIteration
-    
+
     return self.block
 
