@@ -16,7 +16,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2002-2004 Free Software Foundation
+# Copyright 2002-2005 Free Software Foundation
 #
 # FILE:
 # GFContainer.py
@@ -26,8 +26,21 @@
 A base class for all GFObjects that can be containers
 """
 
+from gnue.common import events
+
 from GFObj import GFObj
 
 class GFTabStop(GFObj):
   _navigable = True
-  pass
+
+  def __init__(self, parent, type):
+    GFObj.__init__(self, parent, type)
+    self.subEventHandler = events.EventController()
+
+  def isNavigable(self, mode="edit"):
+    # TODO: Check for read-only, etc
+    if mode == 'query':
+      return self._navigable and self._queryable
+    else:
+      return (self._navigable) and (not self.readonly)
+
