@@ -16,7 +16,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2000-2004 Free Software Foundation
+# Copyright 2000-2005 Free Software Foundation
 #
 # FILE:
 #
@@ -31,6 +31,10 @@ __all__ = [
         "GFOption",
         "GFScrollBar",
         "GFBox",
+        "GFTree",
+        "GFLeaf",
+        "GFColumn",
+        "GFColsrc",
         "GFOptions",
         "GFValue",
         "GFButton",
