@@ -1,3 +1,6 @@
+# GNU Enterprise Forms - Curses UI Driver
+#
+# Copyright 2000-2005 Free Software Foundation
 #
 # This file is part of GNU Enterprise.
 #
@@ -16,20 +19,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2000-2004 Free Software Foundation
-#
-# FILE:
-# __init__.py
-#
-# DESCRIPTION:
-"""
- A curses based user interface driver for GNUe forms.
-"""
-#
-# NOTES:
-#
-
+# $Id: __init__.py 6851 2005-01-03 20:59:28Z jcater $
 
 from UIdriver import GFUserInterface
-from UILoginHandler import *
-
+from UILoginHandler import UILoginHandler
