@@ -1,4 +1,3 @@
-#
 # This file is part of GNU Enterprise.
 #
 # GNU Enterprise is free software; you can redistribute it
@@ -16,10 +15,9 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2000-2004 Free Software Foundation
+# Copyright 2000-2005 Free Software Foundation
 #
-# FILE:
-# win32/widgets/_base.py
+# $Id: _base.py 6963 2005-02-06 12:06:28Z btami $
 #
 # DESCRIPTION:
 # A PyWin32 based user interface driver for GNUe forms.
@@ -38,6 +36,7 @@ from gnue.forms.uidrivers._base.widgets._base import *
 from gnue.forms.uidrivers.win32.common import *
 
 try:
+  import Image
   import ImageWin
 except:
   pass
@@ -69,7 +68,7 @@ class Win32Base:
     self._uiDriver = uidriver
     self._uiDriver._win32app._HwndToTkObj[self._hwnd] = self
 
-    if className in ['STATIC', 'BUTTON', 'COMBOBOX', 'EDIT']:
+    if className in ['STATIC', 'BUTTON', 'COMBOBOX', 'LISTBOX', 'EDIT']:
       self.Show() 
       self._oldWndProc = win32gui.SetWindowLong(self._hwnd, win32con.GWL_WNDPROC, self._wndproc)
 
@@ -77,7 +76,7 @@ class Win32Base:
     if msg == win32con.WM_LBUTTONDOWN:
       gfObject = self._uiDriver._IdToGFObj[self._id]
       uiObject = self._uiDriver._IdToUIObj[self._id]
-  
+
       if 1: #not self.hasFocus():
       # Request Focus
         uiObject._eventHandler('requestFOCUS',gfObject,_form=gfObject._form)
@@ -90,9 +89,9 @@ class Win32Base:
       elif gfObject.style == 'checkbox':
         uiObject._eventHandler('requestTOGGLECHKBOX',_form=gfObject._form)
       else:
-        win32gui.CallWindowProc(self._oldWndProc, hwnd, msg, wParam, lParam)
         cursorPosition = self.getSelectedArea()[1]
         uiObject._eventHandler('requestCURSORMOVE', position=cursorPosition, _form=gfObject._form)
+        win32gui.CallWindowProc(self._oldWndProc, hwnd, msg, wParam, lParam)
 
     elif msg == win32con.WM_LBUTTONUP:
       gfObject = self._uiDriver._IdToGFObj[self._id]
@@ -110,7 +109,8 @@ class Win32Base:
       gfObject = self._uiDriver._IdToGFObj[self._id]
       if (keycode in NOT_WM_CHAR_KEYS):
 
-        if gfObject._type == 'GFButton' or (gfObject._type == 'GFEntry' and gfObject.style != 'dropdown'):
+        if gfObject._type == 'GFButton' or \
+          (gfObject._type == 'GFEntry' and gfObject.style != 'dropdown' and gfObject.style != 'listbox'):
           action = None
 
           ShiftDown = (win32api.GetKeyState(win32con.VK_SHIFT) & 0x8000) and 1 or 0
@@ -157,11 +157,11 @@ class Win32Base:
       #
       # Sigh... a hack for using <enter> in multiline entries
       #
-      if  keycode == 13 and \
+      if keycode == 13 and \
           not ShiftDown and \
           not ControlDown and \
           not AltDown and \
-          int (gConfigForms('enterIsNewLine')) and \
+          gConfigForms('enterIsNewLine') and \
           (hasattr(gfObject,'Char__height') and gfObject.Char__height) > 1:
 
         command = 'NEWLINE'
@@ -184,21 +184,8 @@ class Win32Base:
                      code=10)
 
       elif command and not keycode in NOT_WM_CHAR_KEYS:
-          if keycode == 13 and gfObject._type == 'GFButton':
-            self._connectTable[self._id](self._id)
-          else:
-            action = events.Event('request%s' % command)
+        action = events.Event('request%s' % command)
 
-      elif gfObject._type == 'GFButton':
-        action = events.Event('buttonActivated',gfObject)
-
-      elif gfObject.style == 'checkbox':
-        # <space> <=> <click>
-        if keycode == 32:
-            action = events.Event('requestTOGGLECHKBOX')
-        else:
-          # maybe some background error message here
-          pass
       else:
         try:
           char = chr(keycode)
@@ -219,11 +206,10 @@ class Win32Base:
         uiObject._eventHandler(action)
 
       if gfObject._type == 'GFEntry':
-        if gfObject.style == 'dropdown':
+        if gfObject.style == 'dropdown' or gfObject.style == 'listbox':
           return win32gui.CallWindowProc(self._oldWndProc, hwnd, msg, wParam, lParam)
       
     elif msg == win32con.WM_COMMAND:
-
       if win32api.HIWORD(wParam) == win32con.CBN_SELCHANGE:
         gfObject = self._uiDriver._IdToGFObj[self._id]
         uiObject = self._uiDriver._IdToUIObj[self._id]
@@ -249,7 +235,7 @@ class Win32Base:
       return win32gui.CallWindowProc(self._oldWndProc, hwnd, msg, wParam, lParam)
 
   def SetFont(self,font):
-    self._PyCWnd.SetFont(font,1)
+      self._PyCWnd.SetFont(font,1)
 
   def GetId(self):
     return self._id
@@ -273,7 +259,10 @@ class Win32Base:
     return self._children
 
   def SetFocus(self):
-    win32gui.SetFocus(self._hwnd)
+    try:
+      win32gui.SetFocus(self._hwnd)
+    except:
+      pass
 
   def Enable(self, enabled):
     if enabled:
@@ -320,7 +309,7 @@ class Win32Window(Win32Base):
         win32gui.SendMessage(child.GetHwnd(), commctrl.TB_AUTOSIZE, 0, 0)
     
   def OnWMCommand(self, hwnd, msg, wParam, lParam):
-    # menu/toolbar selection happend
+    # menu/toolbar selection happend'
     self._connectTable[wParam]()
 
   def OnWMNotify(self, hwnd, msg, wParam, lParam):
@@ -356,7 +345,15 @@ class Win32Entry(Win32Base):
           val = win32con.BST_UNCHECKED
         win32gui.SendMessage(self._hwnd, win32con.BM_SETCHECK, val, 0)
       elif gfObject.style == 'dropdown':
-        win32gui.SendMessage(self._hwnd, win32con.CB_SELECTSTRING, -1, value) # CB_SETCURSEL, value, 0)
+        if value:
+          win32gui.SendMessage(self._hwnd, win32con.CB_SELECTSTRING, -1, value)
+        else:
+          win32gui.SendMessage(self._hwnd, win32con.CB_SETCURSEL, 0, 0)
+      elif gfObject.style == 'listbox':
+        if value:
+          win32gui.SendMessage(self._hwnd, win32con.LB_SELECTSTRING, -1, value)
+        else:
+          win32gui.SendMessage(self._hwnd, win32con.LB_SETCURSEL, 0, 0)
       else:
         if hasattr(gfObject,'Char__height') and gfObject.Char__height > 1:
           corrvalue = re.sub(r'(?<!\r)\n', r'\r\n', value)
@@ -376,6 +373,8 @@ class Win32Entry(Win32Base):
         return 0
     elif gfObject.style == 'dropdown':
       return win32gui.SendMessage(self._hwnd, win32con.CB_GETCURSEL, 0, 0)
+    elif gfObject.style == 'listbox':
+      return win32gui.SendMessage(self._hwnd, win32con.LB_GETCURSEL, 0, 0)
     else:
       return win32gui.GetWindowText(self._hwnd)
 
@@ -389,6 +388,7 @@ class Win32Entry(Win32Base):
     gs = win32gui.SendMessage(self._hwnd, win32con.EM_GETSEL, 0, 0)
     return (win32api.LOWORD(gs), win32api.HIWORD(gs))
 
+
 class Win32Button(Win32Base):
   pass
 #  def OnWMCommand(self, hwnd, msg, wParam, lParam):
@@ -400,8 +400,20 @@ class Win32Button(Win32Base):
 class Win32Page(Win32Base):
 
   def OnWMCommand(self, hwnd, msg, wParam, lParam):
-    window = self._uiDriver._win32app._HwndToTkObj[lParam]
-    window.OnWMCommand(hwnd, msg, wParam, lParam)
+    if win32api.HIWORD(wParam) == win32con.LBN_SELCHANGE:
+      lbWindow = self._uiDriver._win32app._HwndToTkObj[lParam]
+      id = lbWindow.GetId()
+      gfObject = self._uiDriver._IdToGFObj[id]
+      uiObject = self._uiDriver._IdToUIObj[id]
+      if gfObject.style == 'listbox':
+        selection = lbWindow.GetValue()
+        string = gfObject._field.allowedValues()[1][selection]
+        uiObject._eventHandler('requestREPLACEVALUE',object=gfObject,
+                            index=selection, text=string,
+                            _form=gfObject._form)
+    else:
+      window = self._uiDriver._win32app._HwndToTkObj[lParam]
+      window.OnWMCommand(hwnd, msg, wParam, lParam)
 
   def OnWMVScroll(self, hwnd, msg, wParam, lParam):
     sbWindow = self._uiDriver._win32app._HwndToTkObj[lParam]
@@ -438,11 +450,43 @@ class Win32Page(Win32Base):
 class Win32Image(Win32Base):
     
   def SetValue(self, value):
-    width, height = value.size
+    l,t,r,b = win32gui.GetWindowRect(self._hwnd)
+    scrx,scry = (r-l, b-t)
+    imgx, imgy = value.size
+    
+    gfObject = self._uiDriver._IdToGFObj[self._id]
+
+    scalex = scaley = 1
+
+    fit = gfObject.fit
+    if fit == "auto":
+      if float(scrx) / imgx < float(scry) / imgy:
+        fit = "width"
+      else:
+        fit = "height"
+
+    if fit == "width":
+      scalex = scaley = float(scrx) / imgx
+    elif fit == "height":
+      scalex = scaley = float(scry) / imgy
+    elif fit == "both":
+      scalex = float(scrx) / imgx
+      scaley = float(scry) / imgy
+
+    if scalex != 1 or scaley != 1:
+      value = value.resize((abs(int(imgx * (scalex))), abs(int(imgy * (scaley)))), Image.BICUBIC)
+
+    # clear the previous image
     win32gui.SetWindowPos(self._hwnd, 0, 0, 0, 0, 0, win32con.SWP_NOMOVE | win32con.SWP_NOZORDER)
-    if width>1:
-      win32gui.SetWindowPos(self._hwnd, 0, 0, 0, width, height, win32con.SWP_NOMOVE | win32con.SWP_NOZORDER)
-      self.dib = ImageWin.Dib(value)
+    
+    # resize the bitmap holder to match the image size again
+    win32gui.SetWindowPos(self._hwnd, 0, 0, 0, scrx, scry, win32con.SWP_NOMOVE | win32con.SWP_NOZORDER)
+    
+    # convert a PIL image to Dib
+    self.dib = ImageWin.Dib(value)
+    
+    # paint it
+    win32gui.SendMessage(self._hwnd, win32con.WM_PAINT, 0, 0)
 
 
 #
@@ -491,16 +535,22 @@ class UIHelper(UIWidget):
 
   def setValue(self, value, index=0, enabled=1):
     widget = self.widgets[index]
-
     # Check if foreign key changed
     gfObject = self._uiDriver._IdToGFObj[widget.GetId()]
     try:
-      if gfObject.style == "dropdown" and \
+      if (gfObject.style == "dropdown" or gfObject.style == "listbox") and \
           not gfObject._field._allowedValues == widget._origAllowedValues:
         widget._origAllowedValues = gfObject._field._allowedValues
-        win32gui.SendMessage(widget.GetHwnd(), win32con.CB_RESETCONTENT, 0, 0)
+        if gfObject.style == "dropdown":
+          RESETCONTENT = win32con.CB_RESETCONTENT
+          ADDSTRING = win32con.CB_ADDSTRING
+        else:
+          RESETCONTENT = win32con.LB_RESETCONTENT
+          ADDSTRING = win32con.LB_ADDSTRING
+
+        win32gui.SendMessage(widget.GetHwnd(), RESETCONTENT, 0, 0)
         for value in gfObject._field.allowedValues()[1]:
-          win32gui.SendMessage(widget.GetHwnd(), win32con.CB_ADDSTRING, 0, textEncode(value.encode))
+          win32gui.SendMessage(widget.GetHwnd(), ADDSTRING, 0, textEncode(value))
         widget.SetValue("")
     except AttributeError:
       pass
@@ -519,9 +569,19 @@ class UIHelper(UIWidget):
     self.widgets[index].setSelectedArea(selection1, selection2)
 
   def createWidget(self, event, spacer):
+    gfObject = event.object
+
+    if hasattr (gfObject, 'Char__y'):
+      posY = gfObject.Char__y
+      gap  = gfObject._gap + 1
+      self.itemY = (posY + spacer * gap) * event.widgetHeight
+
     newWidget = self._createWidget(event, spacer)
     newWidget.SetFont(self._uiDriver._font)
-    self._addToCrossRef(newWidget,event.object,self)
+    if event.initialize:
+      self._eventHandler = event.eventHandler
+      self._addToCrossRef(newWidget,gfObject,self)
+
     return newWidget
 
   def cleanup(self, object):
